/* This header defines a few macros that are useful for porting code
   from other languages (although they are not necessarily very efficient)

   HIGH - Set pin to output and set high (example: HIGH(C,2);)
   LOW - Set pin to output and set low (example: LOW(B,0); )
   TOGGLE - Set pin to output and reverse state
   OUTPUT - Make pin output
   INPUT - Make pin input
   REVERSE - Make input pin an output or output pin an input
   IREAD - Make pin input and return 1 or 0 based on pin state
   BIN - Form a binary number (example: x=BIN(11001001); )

This is part of the APP4 distribution from AWC
http://www.awce.com/app4.htm
*/

#ifndef _AWC_APP4IO_H
#define _AWC_APP4IO_H

#define MKTOKEN(a,b) a ## b
#define HIGH(port,pin) ( MKTOKEN(PORT,port) |= (1<<(pin)), MKTOKEN(DDR,port) |= (1<<(pin)))
#define LOW(port,pin) ( MKTOKEN(PORT,port) &= ~(1<<(pin)), MKTOKEN(DDR,port) |= (1<<(pin)))
#define TOGGLE(port,pin) ( MKTOKEN(PORT,port) ^= (1<<(pin)), MKTOKEN(DDR,port)|= (1<<(pin)))
#define OUTPUT(port,pin) ( MKTOKEN(DDR,port) |= (1<<(pin)))
#define INPUT(port,pin) (MKTOKEN(DDR,port) &= ~(1<<(pin)))
#define REVERSE(port,pin) (MKTOKEN(DDR,port) ^= (1<<(pin)))
#define IREAD(port,pin) (MKTOKEN(DDR, port) &= ~(1<<(pin)),((MKTOKEN(PIN,port)) & (1<<(pin)))!=0)



/* you can use this macro to define a binary number
   x=BIN(11001100);
*/
#define BIN(x) _BIN(0 ## x)
#define _BIN(x) \
((x & 000000001) >> 0)| \
((x & 000000010) >> 2)| \
((x & 000000100) >> 4)| \
((x & 000001000) >> 6)| \
((x & 000010000) >> 8)| \
((x & 000100000) >> 10)| \
((x & 001000000) >> 12)| \
((x & 010000000) >> 14)


#endif
