#include <avr/io.h>
#include "app4.h"
#include "app4delay.h"
#include <avr/pgmspace.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "app4uart.h"
#include "app4adc.h"
#include "app4io.h"
#include "app4lcd.h"

/* dit delay for morse code */
#define DITDELAY 125  /* mS */

/* Output on PORTC.5 */
#define OUTPORT C
#define OUTBIT 5

/* This function does the number of dit delays (roughly) */
void delay(int unit)
{
  int i;
  for (i=0;i<unit;i++) 
    {
    delay_ms(DITDELAY);
    }
}

/* Send a morse code element */
void send(char c)
{
  switch (c)
    {
    case '-':
      HIGH(OUTPORT,OUTBIT);
      delay(3);
      LOW(OUTPORT,OUTBIT);
      break;
    case '.':
      HIGH(OUTPORT,OUTBIT);
      delay(1);
      LOW(OUTPORT,OUTBIT);
      break;
    case ' ':
      delay(4);
      break;
    }
  delay(1);  /* inter element space */
}

int whole,frac;

void doadc(void)
{
  float f=(float)adc_convert(0);
  f=(5*f/1024.0+0.05)*10;
  whole=(int)f;
  frac=whole%10;
  whole/=10;
  printf("%d.%1dV\n",whole,frac);
}

/* Function to send a morse code digit */
void senddigit(int n)
{
  PGM_P digits=PSTR("-----.----..---...--....-.....-....--...---..----.");
  PGM_P mp=digits+n*5;
  int i;
  for (i=0;i<5;i++)
     send(pgm_read_byte(mp++));
  delay(6);
}

/* Main program */
int main (void)
{
  int i;
  char c;
  char str[17];
 UartInit(BAUD_19200);
 UartSetStdio();
 lcd_init(LCD_DISP_ON_CURSOR_BLINK);  
 adc_init();
 printf_P(PSTR("\014APP4 Demo program!\nTurn on local echo for best results\n"));
 printf_P(PSTR("Warning: Do not connect A/D inputs directly to Vcc\n"));
 lcd_puts("APP4");
 while (1) 
   {
     printf_P(PSTR("\n1 - Read A/D converter (ADC0)\n"));
     printf_P(PSTR("2 - Send Morse code message (LED on PC.5)\n"));
     printf_P(PSTR("3 - Message to LCD (see docs for hookup)\n"));
     printf_P(PSTR("Choice: "));
     c=getc(stdin);
     putchar('\n');
     switch (c)
       {
       case '1':
         doadc();
         break;
       case '2':
         LOW(OUTPORT,OUTBIT);   // Make LED an output
         send('-');
         send('.');
         send('-');
         send('.');
         send(' ');
         send('-');         
         send('-');
         send('.');
         send('-');
         send(' ');
         doadc(); // get voltage and send it too
         senddigit(whole);
         for (i=0;i<3;i++) // send a period 
           {
             send('.');
             send('-');  
           }
         send(' ');
         senddigit(frac);
         for (i=0;i<3;i++) send('.'); send('-');  // "V"
         break;
       case '3':
         printf("Message: ");
         fgets(str,sizeof(str),stdin);
         i=strlen(str);
         if (i && str[i-1]=='\n') str[i-1]='\0';
         lcd_clear();
         lcd_puts(str);
         break;
       default:
         putchar('?');
         putchar('\n');
       }
  }
  return 0;
}

